@@ -1,18 +1,19 @@
 import logging
 
-from typing import TYPE_CHECKING, Optional, Dict, Any, Tuple, List, Union, Type
+from typing import TYPE_CHECKING, Optional, Dict, Any, Tuple, List, Type, cast
 from os import environ, path
 from dataclasses import dataclass, field
 
 import yaml
 
 from behave.model import Scenario
-from locust.env import Environment
+from locust.runners import Runner
 
-from .types import GrizzlyDict
+from .types import MessageCallback, MessageDirection
+from .testdata import GrizzlyVariables
 
 if TYPE_CHECKING:  # pragma: no cover
-    from .tasks import GrizzlyTask, AsyncRequestGroupTask
+    from .tasks import GrizzlyTask, AsyncRequestGroupTask, TimerTask
 
 
 logger = logging.getLogger(__name__)
@@ -53,21 +54,65 @@ def load_configuration_file() -> Dict[str, Any]:
         raise SystemExit(1)
 
 
+class GrizzlyContext:
+    __instance: Optional['GrizzlyContext'] = None
+
+    _initialized: bool
+    _state: 'GrizzlyContextState'
+    _setup: 'GrizzlyContextSetup'
+    _scenarios: 'GrizzlyContextScenarios'
+
+    @classmethod
+    def __new__(cls, *args: Tuple[Any, ...], **kwargs: Dict[str, Any]) -> 'GrizzlyContext':
+        if cls.__instance is None:
+            cls.__instance = super().__new__(cls)
+            cls.__instance._initialized = False
+
+        return cls.__instance
+
+    @classmethod
+    def destroy(cls) -> None:
+        if cls.__instance is None:
+            raise ValueError(f"'{cls.__name__}' is not instantiated")
+
+        cls.__instance = None
+
+    def __init__(self) -> None:
+        if not self._initialized:
+            self._state = GrizzlyContextState()
+            self._setup = GrizzlyContextSetup()
+            self._scenarios = GrizzlyContextScenarios()
+            self._initialized = True
+
+    @property
+    def setup(self) -> 'GrizzlyContextSetup':
+        return self._setup
+
+    @property
+    def state(self) -> 'GrizzlyContextState':
+        return self._state
+
+    @property
+    def scenario(self) -> 'GrizzlyContextScenario':
+        if len(self._scenarios) < 1:
+            self._scenarios.append(GrizzlyContextScenario(1))
+
+        return self._scenarios[-1]
+
+    @property
+    def scenarios(self) -> 'GrizzlyContextScenarios':
+        return self._scenarios
+
+
 @dataclass
 class GrizzlyContextState:
     spawning_complete: bool = field(default=False)
     background_section_done: bool = field(default=False)
-    variables: GrizzlyDict = field(init=False, default_factory=GrizzlyDict)
+    variables: GrizzlyVariables = field(init=False, default_factory=GrizzlyVariables)
     configuration: Dict[str, Any] = field(init=False, default_factory=load_configuration_file)
     alias: Dict[str, str] = field(init=False, default_factory=dict)
     verbose: bool = field(default=False)
-    environment: Environment = field(init=False, repr=False)
-
-
-@dataclass
-class GrizzlyContextScenarioWait:
-    minimum: float = field(default=1.0)
-    maximum: float = field(default=1.0)
+    locust: Runner = field(init=False, repr=False)
 
 
 @dataclass
@@ -89,9 +134,30 @@ class GrizzlyContextScenarioUser:
     weight: int = field(init=False, hash=True, default=1)
 
 
+class GrizzlyContextTasks(List['GrizzlyTask']):
+    scenario: 'GrizzlyContextScenario'
+
+    def __init__(self, scenario: 'GrizzlyContextScenario', *args: Tuple[Any, ...], **kwargs: Dict[str, Any]) -> None:
+        super().__init__(*args, **kwargs)
+
+        self.scenario = scenario
+
+    def __call__(self) -> List['GrizzlyTask']:
+        return cast(List['GrizzlyTask'], self)
+
+    def add(self, task: 'GrizzlyTask', pos: Optional[int] = None) -> None:
+        if not hasattr(task, 'scenario') or task.scenario is None or task.scenario is not self.scenario:
+            task.scenario = self.scenario
+
+        if pos is None:
+            self.append(task)
+        else:
+            self.insert(pos, task)
+
+
 @dataclass(unsafe_hash=True)
 class GrizzlyContextScenario:
-    name: str = field(init=False, hash=True)
+    _name: str = field(init=False, hash=True)
     description: str = field(init=False, hash=False)
     user: GrizzlyContextScenarioUser = field(init=False, hash=False, compare=False, default_factory=GrizzlyContextScenarioUser)
     index: int = field(init=True)
@@ -99,18 +165,41 @@ class GrizzlyContextScenario:
 
     behave: Scenario = field(init=False, repr=False, hash=False, compare=False)
     context: Dict[str, Any] = field(init=False, repr=False, hash=False, compare=False, default_factory=dict)
-    wait: GrizzlyContextScenarioWait = field(init=False, repr=False, hash=False, compare=False, default_factory=GrizzlyContextScenarioWait)
-    tasks: List['GrizzlyTask'] = field(init=False, repr=False, hash=False, compare=False, default_factory=list)
+    _tasks: GrizzlyContextTasks = field(init=False, repr=False, hash=False, compare=False)
     validation: GrizzlyContextScenarioValidation = field(init=False, hash=False, compare=False, default_factory=GrizzlyContextScenarioValidation)
     failure_exception: Optional[Type[Exception]] = field(init=False, default=None)
     orphan_templates: List[str] = field(init=False, repr=False, hash=False, compare=False, default_factory=list)
     async_group: Optional['AsyncRequestGroupTask'] = field(init=False, repr=False, hash=False, compare=False, default=None)
+    timers: Dict[str, Optional['TimerTask']] = field(init=False, repr=False, hash=False, compare=False, default_factory=dict)
+
+    def __post_init__(self) -> None:
+        self._tasks = GrizzlyContextTasks(self)
+
+    @property
+    def tasks(self) -> GrizzlyContextTasks:
+        return self._tasks
 
     @property
     def identifier(self) -> str:
         return f'{self.index:03}'
 
-    def get_name(self) -> str:
+    @property
+    def name(self) -> str:
+        if self._name.endswith(f'_{self.identifier}'):
+            return self._name.replace(f'_{self.identifier}', '')
+        else:
+            return self._name
+
+    @name.setter
+    def name(self, value: str) -> None:
+        self._name = value
+
+    @property
+    def locust_name(self) -> str:
+        return f'{self.identifier} {self.description}'
+
+    @property
+    def class_name(self) -> str:
         if not self.name.endswith(f'_{self.identifier}'):
             return f'{self.name}_{self.identifier}'
         else:
@@ -123,11 +212,21 @@ class GrizzlyContextScenario:
             or self.validation.response_time_percentile is not None
         )
 
-    def add_task(self, task: 'GrizzlyTask') -> None:
-        if not hasattr(task, 'scenario') or task.scenario is None or task.scenario is not self:
-            task.scenario = self
 
-        self.tasks.append(task)
+class GrizzlyContextSetupLocustMessages(Dict[MessageDirection, Dict[str, MessageCallback]]):
+    def register(self, direction: MessageDirection, message_type: str, callback: MessageCallback) -> None:
+        if direction not in self:
+            self[direction] = {}
+
+        if message_type not in self[direction]:
+            self[direction][message_type] = callback
+        else:
+            raise AttributeError(f'message type {message_type} was already registered in direction {direction.name}')
+
+
+@dataclass
+class GrizzlyContextSetupLocust:
+    messages: GrizzlyContextSetupLocustMessages = field(default_factory=GrizzlyContextSetupLocustMessages)
 
 
 @dataclass
@@ -142,70 +241,33 @@ class GrizzlyContextSetup:
 
     statistics_url: Optional[str] = field(init=False, default=None)
 
+    locust: GrizzlyContextSetupLocust = field(init=False, default_factory=GrizzlyContextSetupLocust)
 
-class GrizzlyContext:
-    __instance: Optional['GrizzlyContext'] = None
 
-    _initialized: bool
-    _state: GrizzlyContextState
-    _setup: GrizzlyContextSetup
-    _scenarios: List[GrizzlyContextScenario]
+class GrizzlyContextScenarios(List[GrizzlyContextScenario]):
+    def __call__(self) -> List[GrizzlyContextScenario]:
+        return cast(List[GrizzlyContextScenario], self)
 
-    @classmethod
-    def __new__(cls, *_args: Tuple[Any, ...], **_kwargs: Dict[str, Any]) -> 'GrizzlyContext':
-        if cls.__instance is None:
-            cls.__instance = super().__new__(cls)
-            cls.__instance._initialized = False
+    def find_by_class_name(self, class_name: str) -> Optional[GrizzlyContextScenario]:
+        return self._find(class_name, 'class_name')
 
-        return cls.__instance
+    def find_by_name(self, name: str) -> Optional[GrizzlyContextScenario]:
+        return self._find(name, 'name')
 
-    @classmethod
-    def destroy(cls) -> None:
-        if cls.__instance is None:
-            raise ValueError(f"'{cls.__name__}' is not instantiated")
+    def find_by_description(self, description: str) -> Optional[GrizzlyContextScenario]:
+        return self._find(description, 'description')
 
-        cls.__instance = None
-
-    def __init__(self) -> None:
-        if not self._initialized:
-            self._state = GrizzlyContextState()
-            self._setup = GrizzlyContextSetup()
-            self._scenarios = []
-            self._initialized = True
-
-    @property
-    def setup(self) -> GrizzlyContextSetup:
-        return self._setup
-
-    @property
-    def state(self) -> GrizzlyContextState:
-        return self._state
-
-    @property
-    def scenario(self) -> GrizzlyContextScenario:
-        if len(self._scenarios) < 1:
-            self._scenarios.append(GrizzlyContextScenario(1))
-
-        return self._scenarios[-1]
-
-    def add_scenario(self, source: Union[Scenario, str]) -> None:
-        scenario = GrizzlyContextScenario(len(self._scenarios) + 1)
-        if isinstance(source, Scenario):
-            name = source.name
-            scenario.behave = source
-        else:
-            name = source
-
-        scenario.name = name
-        scenario.description = name
-        self._scenarios.append(scenario)
-
-    def scenarios(self) -> List[GrizzlyContextScenario]:
-        return self._scenarios
-
-    def get_scenario(self, name: str) -> Optional[GrizzlyContextScenario]:
-        for scenario in self._scenarios:
-            if scenario.get_name() == name:
-                return scenario
+    def _find(self, value: str, attribute: str) -> Optional[GrizzlyContextScenario]:
+        for item in self:
+            if getattr(item, attribute, None) == value:
+                return item
 
         return None
+
+    def create(self, behave_scenario: Scenario) -> None:
+        grizzly_scenario = GrizzlyContextScenario(len(self) + 1)
+        grizzly_scenario.behave = behave_scenario
+        grizzly_scenario.name = behave_scenario.name
+        grizzly_scenario.description = behave_scenario.name
+
+        self.append(grizzly_scenario)
