@@ -2,23 +2,28 @@ import sys
 import logging
 import subprocess
 
-from typing import Optional, Callable, List, Tuple, Set, Dict, Type, cast
+from typing import NoReturn, Optional, Callable, List, Tuple, Set, Dict, Type, Union, cast
 from os import environ
 from signal import SIGTERM
 from socket import error as SocketError
 from datetime import datetime
 from math import ceil
+from operator import itemgetter
 
 import gevent
 
 from behave.runner import Context
+from behave.model import Status
 from locust.runners import MasterRunner, WorkerRunner, Runner
 from locust.env import Environment
 from locust.stats import (
+    RequestStats,
+    CONSOLE_STATS_INTERVAL_SEC,
+    STATS_NAME_WIDTH,
+    STATS_TYPE_WIDTH,
     print_error_report,
     print_percentile_stats,
     print_stats,
-    stats_printer,
     stats_history,
 )
 from locust.log import setup_logging
@@ -28,7 +33,7 @@ from jinja2.exceptions import TemplateError
 
 from .listeners import init, init_statistics_listener, quitting, validate_result, spawning_complete, locust_test_start, locust_test_stop
 from .testdata.utils import initialize_testdata
-from .types import TestdataType
+from .types import RequestType, TestdataType
 from .context import GrizzlyContext
 from .tasks import GrizzlyTask
 from .users.base import GrizzlyUser
@@ -43,6 +48,8 @@ unhandled_greenlet_exception = False
 
 logger = logging.getLogger('grizzly.locust')
 
+stats_logger = logging.getLogger('locust.stats_logger')
+
 
 def greenlet_exception_logger(logger: logging.Logger, level: int = logging.CRITICAL) -> Callable[[gevent.Greenlet], None]:
     def exception_handler(greenlet: gevent.Greenlet) -> None:
@@ -134,6 +141,10 @@ def setup_locust_scenarios(grizzly: GrizzlyContext) -> Tuple[List[Type[GrizzlyUs
             scenario_type.populate(task)
             tasks.append(task)
 
+            dependencies = getattr(task, '__dependencies__', None)
+            if dependencies is not None:
+                external_dependencies.update(dependencies)
+
         logger.debug(
             f'{user_class_type.__name__}/{scenario_type.__name__}: tasks={len(scenario.tasks)}, weight={user_class_type.weight}, fixed_count={user_class_type.fixed_count}'
         )
@@ -164,7 +175,11 @@ def setup_resource_limits(context: Context) -> None:
             )
 
 
-def setup_environment_listeners(context: Context, environment: Environment, tasks: List[GrizzlyTask]) -> Set[str]:
+def setup_environment_listeners(context: Context, tasks: List[GrizzlyTask]) -> Set[str]:
+    grizzly = cast(GrizzlyContext, context.grizzly)
+
+    environment = grizzly.state.locust.environment
+
     # make sure we don't have any listeners
     environment.events.init._handlers = []
     environment.events.test_start._handlers = []
@@ -172,15 +187,13 @@ def setup_environment_listeners(context: Context, environment: Environment, task
     environment.events.spawning_complete._handlers = []
     environment.events.quitting._handlers = []
 
-    grizzly = cast(GrizzlyContext, context.grizzly)
-
     # add standard listeners
     testdata: Optional[TestdataType] = None
     external_dependencies: Set[str] = set()
 
     # initialize testdata
     try:
-        testdata, external_dependencies = initialize_testdata(tasks)
+        testdata, external_dependencies = initialize_testdata(grizzly, tasks)
 
         for scenario_testdata in testdata.values():
             for variable, value in scenario_testdata.items():
@@ -205,7 +218,7 @@ def setup_environment_listeners(context: Context, environment: Environment, task
 
         environment.events.quitting.add_listener(quitting)
 
-    environment.events.init.add_listener(init(testdata))
+    environment.events.init.add_listener(init(grizzly, testdata))
     environment.events.test_start.add_listener(locust_test_start(grizzly))
     environment.events.test_stop.add_listener(locust_test_stop)
 
@@ -216,49 +229,64 @@ def setup_environment_listeners(context: Context, environment: Environment, task
     if grizzly.setup.statistics_url is not None:
         environment.events.init.add_listener(init_statistics_listener(grizzly.setup.statistics_url))
 
-    grizzly.state.environment = environment
-
     return external_dependencies
 
 
 def print_scenario_summary(grizzly: GrizzlyContext) -> None:
-    def print_table_lines(max_length_iterations: int, max_length_description: int) -> None:
-        sys.stdout.write('-' * 5)
-        sys.stdout.write('-|-')
-        sys.stdout.write('-' * max_length_iterations)
-        sys.stdout.write('|-')
-        sys.stdout.write('-' * max_length_description)
-        sys.stdout.write('-|\n')
+    def create_separator(max_length_iterations: int, max_length_status: int, max_length_description: int) -> str:
+        separator: List[str] = []
+        separator.append('-' * 5)
+        separator.append('-|-')
+        separator.append('-' * max_length_iterations)
+        separator.append('|-')
+        separator.append('-' * max_length_status)
+        separator.append('-|-')
+        separator.append('-' * max_length_description)
+        separator.append('-|')
+
+        return ''.join(separator)
 
     rows: List[str] = []
     max_length_description = len('description')
-    max_length_iterations = len('#')
+    max_length_iterations = len('iter')
+    max_length_status = len('status')
+
+    stats = grizzly.state.locust.environment.stats
 
     for scenario in grizzly.scenarios():
-        description_length = len(scenario.description or 'unknown')
-        if description_length > max_length_description:
-            max_length_description = description_length
-
-        iterations_length = len(str(scenario.iterations or ''))
-        if iterations_length > max_length_iterations:
-            max_length_iterations = iterations_length
+        stat = stats.get(scenario.locust_name, RequestType.SCENARIO())
+        max_length_description = max(len(scenario.description or 'unknown'), max_length_description)
+        max_length_iterations = max(len(f'{stat.num_requests}/{scenario.iterations or 0}'), max_length_iterations)
+        max_length_status = max(len(Status.undefined.name) if stat.num_requests < 1 else len(Status.passed.name), max_length_status)
 
     for scenario in grizzly.scenarios():
+        stat = stats.get(scenario.locust_name, RequestType.SCENARIO())
+        if stat.num_requests > 0:
+            if stat.num_failures == 0 and stat.num_requests == scenario.iterations:
+                status = Status.passed
+            else:
+                status = Status.failed
+        else:
+            status = Status.undefined
+
         description = scenario.description or 'unknown'
-        row = '{:5}   {:>{}}  {}'.format(
+        row = '{:5}   {:>{}}  {:{}}   {}'.format(
             scenario.identifier,
-            scenario.iterations,
+            f'{stat.num_requests}/{scenario.iterations}',
             max_length_iterations,
+            status.name,
+            max_length_status,
             description,
         )
         rows.append(row)
 
     print('Scenario')
-    print('{:5}   {:>{}}  {}'.format('ident', '#', max_length_iterations, 'description'))
-    print_table_lines(max_length_iterations, max_length_description)
+    print('{:5}   {:>{}}  {:{}}   {}'.format('ident', 'iter', max_length_iterations, 'status', max_length_status, 'description'))
+    separator = create_separator(max_length_iterations, max_length_status, max_length_description)
+    print(separator)
     for row in rows:
         print(row)
-    print_table_lines(max_length_iterations, max_length_description)
+    print(separator)
 
 
 def run(context: Context) -> int:
@@ -271,6 +299,7 @@ def run(context: Context) -> int:
                 logger.debug(f'{process.returncode=}')
 
             processes.clear()
+
     grizzly = cast(GrizzlyContext, context.grizzly)
 
     log_level = 'DEBUG' if context.config.verbose else grizzly.setup.log_level
@@ -310,25 +339,6 @@ def run(context: Context) -> int:
             stop_timeout=300,  # only wait at most?
         )
 
-        variable_dependencies = setup_environment_listeners(context, environment, tasks)
-        external_dependencies.update(variable_dependencies)
-
-        if not on_master(context) and len(external_dependencies) > 0:
-            env = environ.copy()
-            if grizzly.state.verbose:
-                env['GRIZZLY_EXTRAS_LOGLEVEL'] = 'DEBUG'
-
-            for external_dependency in external_dependencies:
-                logger.info(f'starting {external_dependency}')
-                external_processes.update({external_dependency: subprocess.Popen(
-                    [external_dependency],
-                    env=env,
-                    shell=False,
-                    stdout=subprocess.DEVNULL,
-                    stderr=subprocess.DEVNULL,
-                )})
-                gevent.sleep(2)
-
         runner: Runner
 
         if on_master(context):
@@ -355,6 +365,27 @@ def run(context: Context) -> int:
         else:
             runner = environment.create_local_runner()
 
+        grizzly.state.locust = runner
+
+        variable_dependencies = setup_environment_listeners(context, tasks)
+        external_dependencies.update(variable_dependencies)
+
+        if not on_master(context) and len(external_dependencies) > 0:
+            env = environ.copy()
+            if grizzly.state.verbose:
+                env['GRIZZLY_EXTRAS_LOGLEVEL'] = 'DEBUG'
+
+            for external_dependency in external_dependencies:
+                logger.info(f'starting {external_dependency}')
+                external_processes.update({external_dependency: subprocess.Popen(
+                    [external_dependency],
+                    env=env,
+                    shell=False,
+                    stdout=subprocess.DEVNULL,
+                    stderr=subprocess.DEVNULL,
+                )})
+                gevent.sleep(2)
+
         main_greenlet = runner.greenlet
 
         # And run for maximum
@@ -407,7 +438,7 @@ def run(context: Context) -> int:
             logger.info('starting locust via grizzly')
             runner.start(grizzly.setup.user_count, grizzly.setup.spawn_rate)
 
-            stats_printer_greenlet = gevent.spawn(stats_printer(environment.stats))
+            stats_printer_greenlet = gevent.spawn(grizzly_stats_printer(environment.stats))
             stats_printer_greenlet.link_exception(greenlet_exception_handler)
 
         def spawn_run_time_limit_greenlet() -> None:
@@ -455,7 +486,7 @@ def run(context: Context) -> int:
                 if watch_running_users_greenlet is not None:
                     watch_running_users_greenlet.kill(block=False)
 
-                print_stats(runner.stats, current=False)
+                grizzly_print_stats(runner.stats, current=False)
                 print_percentile_stats(runner.stats)
                 print_error_report(runner.stats)
                 print_scenario_summary(grizzly)
@@ -509,3 +540,56 @@ def run(context: Context) -> int:
             return code
     finally:
         shutdown_external_processes(external_processes)
+
+
+def grizzly_stats_printer(stats: RequestStats) -> Callable[[], NoReturn]:
+    def _grizzly_stats_printer() -> NoReturn:
+        while True:
+            grizzly_print_stats(stats)
+            gevent.sleep(CONSOLE_STATS_INTERVAL_SEC)
+
+    return _grizzly_stats_printer
+
+
+def grizzly_print_stats(stats: RequestStats, current: bool = True, grizzly_style: bool = True) -> None:
+    if not grizzly_style:
+        print_stats(stats, current=current)
+        return
+
+    name_column_width = (STATS_NAME_WIDTH - STATS_TYPE_WIDTH) + 4  # saved characters by compacting other columns
+    stats_logger.info(
+        ("%-" + str(STATS_TYPE_WIDTH) + "s %-" + str(name_column_width) + "s %7s %12s |%7s %7s %7s%7s | %7s %11s")
+        % ("Type", "Name", "# reqs", "# fails", "Avg", "Min", "Max", "Med", "req/s", "failures/s")
+    )
+    separator = f'{"-" * STATS_TYPE_WIDTH}|{"-" * (name_column_width)}|{"-" * 7}|{"-" * 13}|{"-" * 7}|{"-" * 7}|{"-" * 7}|{"-" * 7}|{"-" * 8}|{"-" * 11}'
+    stats_logger.info(separator)
+
+    keys: List[Union[Tuple[str, str], Tuple[str, str, int]]] = sorted(stats.entries.keys())
+
+    previous_ident: Optional[str] = None
+    scenario_keys: List[Tuple[str, str]] = []
+    scenario_sorted_keys: List[Tuple[str, str, int]] = []
+    for index, key in enumerate(keys):
+        ident, _ = key[0].split(' ', 1)
+        is_last = index == len(keys) - 1
+        if (previous_ident is not None and previous_ident != ident) or is_last:
+            if is_last:
+                scenario_keys.append(key[:2])
+
+            scenario_sorted_keys += sorted([
+                (name, method, RequestType.get_method_weight(method), ) for name, method in scenario_keys
+            ], key=itemgetter(2, 0))
+            scenario_keys.clear()
+
+        previous_ident = ident
+        scenario_keys.append(key[:2])
+
+    keys = cast(List[Union[Tuple[str, str], Tuple[str, str, int]]], scenario_sorted_keys)
+
+    for key in keys:
+        r = stats.entries[key[:2]]
+        stats_logger.info(r.to_string(current=current))
+
+    stats_logger.info(separator)
+    stats_logger.info(stats.total.to_string(current=current))
+    stats_logger.info('')
