@@ -1,16 +1,33 @@
-'''This module contains step implementations that configures the load test scenario with parameters applicable for all scenarios.'''
+'''
+@anchor pydoc:grizzly.steps.background.setup Setup
+This module contains step implementations that configures the load test scenario with parameters applicable for all scenarios.'''
+import parse
 
 from urllib.parse import urlparse, parse_qs, urlunparse
 from typing import cast, List
+from importlib import import_module
+from inspect import signature
 
-from behave import given  # pylint: disable=no-name-in-module
+from behave import register_type, given  # pylint: disable=no-name-in-module
 from behave.runner import Context
 
+from grizzly.types import MessageDirection, Environment, Message
+
 from ...context import GrizzlyContext
 from ...utils import merge_dicts
 from ...testdata.utils import create_context_variable, resolve_variable
 
 
+@parse.with_pattern(r'(client|server)', regex_group_count=1)
+def parse_message_direction(text: str) -> str:
+    return text.strip()
+
+
+register_type(
+    MessageDirection=parse_message_direction,
+)
+
+
 @given(u'save statistics to "{url}"')
 def step_setup_save_statistics(context: Context, url: str) -> None:
     '''Sets an URL where locust statistics should be sent.
@@ -19,19 +36,19 @@ def step_setup_save_statistics(context: Context, url: str) -> None:
 
     For InfluxDB the following format **must** be used:
 
-    ```plain
+    ``` plain
     influxdb://[<username>:<password>@]<hostname>[:<port>]/<database>?TargetEnviroment=<target environment>[&Testplan=<test plan>]
     [&TargetEnvironment=<target environment>][&ProfileName=<profile name>][&Description=<description>]
     ```
 
     For Azure Application Insights the following format **must** be used:
 
-    ```plain
+    ``` plain
     insights://?InstrumentationKey=<instrumentation key>&IngestionEndpoint=<ingestion endpoint>[&Testplan=<test plan>]
     insights://<ingestion endpoint>/?InstrumentationKey=<instrumentation key>[&Testplan=<test plan>]
     ```
 
-    ```gherkin
+    ``` gherkin
     And save statistics to "influxdb://grizzly:secret-password@influx.example.com/grizzly-statistics"
     And save statistics to "insights://?IngestionEndpoint=https://insights.example.com&Testplan=grizzly-statistics&InstrumentationKey=asdfasdfasdf="
     And save statistics to "insights://insights.example.com/?Testplan=grizzly-statistics&InstrumentationKey=asdfasdfasdf="
@@ -96,7 +113,7 @@ def step_setup_log_level(context: Context, log_level: str) -> None:
 
     Default value is `INFO`, by changing to `DEBUG` there is more information what `grizzly` is doing behind the curtains.
 
-    ```gherkin
+    ``` gherkin
     And log level is "DEBUG"
     ```
 
@@ -113,7 +130,7 @@ def step_setup_run_time(context: Context, timespan: str) -> None:
     '''Configures the time period a headless test should run for.
     If available test data is infinite, the test will run forever if this step is not used.
 
-    ```gherkin
+    ``` gherkin
     And run for maximum "1h"
     ```
 
@@ -136,7 +153,7 @@ def step_setup_set_global_context_variable(context: Context, variable: str, valu
 
     E.g. `token.url` and `token/URL` results in:
 
-    ```json
+    ``` json
     {
         'token': {
             'url': '<value>'
@@ -148,8 +165,8 @@ def step_setup_set_global_context_variable(context: Context, variable: str, valu
 
     E.g. `Client ID` results in `client_id`.
 
-    ```gherkin
-    And set global context variable "token.url" to "http://test.nu/api/auth"
+    ``` gherkin
+    And set global context variable "token.url" to "http://example.com/api/auth"
     And set global context variable "token/client_id" to "aaaa-bbbb-cccc-dddd"
     And set global context variable "token/client secret" to "aasdfasdfasdf=="
     And set global context variable "token.resource" to "0000-aaaaaaa-1111-1111-1111"
@@ -161,7 +178,7 @@ def step_setup_set_global_context_variable(context: Context, variable: str, valu
     Data type of values will be guessed, if not explicitly specified by the type of variable used (`Atomic*`). E.g. the last two
     examples above will result in:
 
-    ```json
+    ``` json
     {
         'validate_certificates': False,
         'run_id': 13
@@ -176,3 +193,41 @@ def step_setup_set_global_context_variable(context: Context, variable: str, valu
     context_variable = create_context_variable(grizzly, variable, value)
 
     grizzly.setup.global_context = merge_dicts(grizzly.setup.global_context, context_variable)
+
+
+@given(u'add callback "{callback_name}" for message type "{message_type}" from {from_node:MessageDirection} to {to_node:MessageDirection}')
+def step_setup_message_type_callback(context: Context, callback_name: str, message_type: str, from_node: str, to_node: str) -> None:
+    grizzly = cast(GrizzlyContext, context.grizzly)
+
+    assert from_node != to_node, f'cannot register message handler that sends from {from_node} and is received at {to_node}'
+
+    message_direction = MessageDirection.from_string(f'{from_node}_{to_node}')
+
+    module_name, callback_name = callback_name.rsplit('.', 1)
+
+    try:
+        module = import_module(module_name)
+    except ModuleNotFoundError as e:
+        assert 0, f'no module named {e.name}'
+
+    callback = getattr(module, callback_name, None)
+
+    assert callback is not None, f'module {module_name} has no method {callback_name}'
+    assert callable(callback), f'{module_name}.{callback_name} is not a method'
+
+    method_signature = signature(callback)
+    parameters = method_signature.parameters
+    parameter_names = list(method_signature.parameters.keys())
+
+    correct_signature = (
+        len(parameter_names) >= 2
+        and parameter_names[0] == 'environment'
+        and issubclass(parameters['environment'].annotation, Environment)
+        and parameter_names[1] == 'msg'
+        and issubclass(parameters['msg'].annotation, Message)
+        and method_signature.return_annotation is None
+    )
+
+    assert correct_signature, f'{module_name}.{callback_name} does not have grizzly.types.MessageCallback method signature: {method_signature}'
+
+    grizzly.setup.locust.messages.register(message_direction, message_type, callback)
