@@ -1,13 +1,13 @@
-'''This module contains step implementations that describes requests sent by `user_class_name` targeting `host`.'''
+'''This module contains step implementations that creates requests executed by the specified {@pylink grizzly.users} in the scenario.'''
 from typing import cast
 
 from behave.runner import Context
 from behave import register_type, then, given  # pylint: disable=no-name-in-module
 
-from ..helpers import add_request_task, get_task_client, is_template
+from .._helpers import add_request_task, get_task_client, is_template
 from ...types import RequestDirection, RequestMethod
 from ...context import GrizzlyContext
-from ...tasks import PrintTask, WaitTask, TransformerTask, UntilRequestTask, DateTask, AsyncRequestGroupTask
+from ...tasks import LogMessage, WaitTask, TransformerTask, UntilRequestTask, DateTask, AsyncRequestGroupTask, TimerTask, RequestWaitTask
 
 from grizzly_extras.transformer import TransformerContentType
 
@@ -28,29 +28,22 @@ register_type(
 
 
 @then(u'{method:Method} request with name "{name}" from endpoint "{endpoint}" until "{condition}"')
-def step_task_request_with_name_to_endpoint_until(context: Context, method: RequestMethod, name: str, endpoint: str, condition: str) -> None:
-    '''Creates a named request to an endpoint on `host` and repeat it until `condition` is true in the response.
+def step_task_request_with_name_endpoint_until(context: Context, method: RequestMethod, name: str, endpoint: str, condition: str) -> None:
+    '''
+    Creates an instance of the {@pylink grizzly.tasks.until} task, see task documentation for more information.
 
-    ```gherkin
+    Example:
+
+    ``` gherkin
     Then get request with name "test-get" from endpoint "/api/test | content_type=json" until "$.`this`[?success==true]"
     Then receive request with name "test-receive" from endpoint "queue:receive-queue | content_type=xml" until "/header/success[. == 'True']"
     ```
 
-    `content_type` will be removed from the actual `endpoint` value.
-
-    `condition` is a JSON- or Xpath expression, that also has support for "grizzly style" arguments:
-
     Args:
         method (RequestMethod): type of request
         name (str): name of the requests in logs, can contain variables
         direction (RequestDirection): one of `to` or `from` depending on the value of `method`
         endpoint (str): URI relative to `host` in the scenario, can contain variables and in certain cases `user_class_name` specific parameters
-
-    ## Arguments:
-
-    * `retries` (int): maximum number of times to repeat the request if `condition` is not met (default `3`)
-
-    * `wait` (float): number of seconds to wait between retries (default `1.0`)
     '''
 
     assert method.direction == RequestDirection.FROM, 'this step is only valid for request methods with direction FROM'
@@ -67,27 +60,33 @@ def step_task_request_with_name_to_endpoint_until(context: Context, method: Requ
         for key, value in substitues.items():
             condition_rendered = condition_rendered.replace(f'{{{{ {key} }}}}', value)
 
-        grizzly.scenario.add_task(UntilRequestTask(
+        grizzly.scenario.tasks.add(UntilRequestTask(
+            grizzly,
             request=request_task,
             condition=condition_rendered,
         ))
 
 
 @then(u'{method:Method} request with name "{name}" {direction:Direction} endpoint "{endpoint}"')
-def step_task_request_text_with_name_to_endpoint(context: Context, method: RequestMethod, name: str, direction: RequestDirection, endpoint: str) -> None:
-    '''Creates a named request to an endpoint on `host`, where optional payload is defined directly in the feature file.
+def step_task_request_text_with_name_endpoint(context: Context, method: RequestMethod, name: str, direction: RequestDirection, endpoint: str) -> None:
+    '''
+    Creates an instance of the {@pylink grizzly.tasks.request} task, where optional payload is defined directly in the feature file.
+    See {@pylink grizzly.tasks.request} task documentation for more information about arguments.
 
-    If `method` in the expression is `get` or `receive`; the `direction` **must** be `from`.
-    If `method` in the expression is `post`, `pust`, or `send`; the `direction` **must** be `to`, and payload defined in the feature file.
+    * If `method` in the expression is `get` or `receive`; the `direction` **must** be `from`.
 
-    ```gherkin
+    * If `method` in the expression is `post`, `pust`, or `send`; the `direction` **must** be `to`, and payload defined in the feature file.
+
+    Example:
+
+    ``` gherkin
     Then post request with name "test-post" to endpoint "/api/test"
         """
         {
             "test": "hello world"
         }
         """
-    Then put request with name "test-post" to endpoint "/api/test"
+    Then put request with name "test-put" to endpoint "/api/test"
         """
         {
             "test": "hello world"
@@ -104,18 +103,6 @@ def step_task_request_text_with_name_to_endpoint(context: Context, method: Reque
     Then receive request with name "test-receive" from endpoint "queue:receive-queue"
     ```
 
-    `endpoint` has support for setting response content type as a parameter:
-
-    ```gherkin
-    Then receive request with name "test-receive" from endpoint "queue:receive-queue | content_type=xml"
-
-    # same as
-    Then receive request with name "test-receive" from endpoint "queue:receive-queue"
-    And set response content type to "xml"
-    ```
-
-    `content_type` will be removed from the actual `endpoint` value.
-
     Args:
         method (RequestMethod): type of request
         name (str): name of the requests in logs, can contain variables
@@ -137,26 +124,19 @@ def step_task_request_text_with_name_to_endpoint(context: Context, method: Reque
 
 @then(u'{method:Method} request "{source}" with name "{name}" to endpoint "{endpoint}"')
 def step_task_request_file_with_name_endpoint(context: Context, method: RequestMethod, source: str, name: str, endpoint: str) -> None:
-    '''Creates a named request to an endpoint on `host`, where the payload is defined in a template file.
+    '''
+    Creates an instance of the {@pylink grizzly.tasks.request} task, where the payload is defined in a template file.
 
-    ```gherkin
+    See {@pylink grizzly.tasks.request} task documentation for more information about arguments.
+
+    Example:
+
+    ``` gherkin
     Then send request "test/request.j2.json" with name "test-send" to endpoint "queue:receive-queue"
     Then post request "test/request.j2.json" with name "test-post" to endpoint "/api/test"
     Then put request "test/request.j2.json" with name "test-put" to endpoint "/api/test"
     ```
 
-    `endpoint` has support for setting response content type as a parameter:
-
-    ```gherkin
-    Then put request "test/request.j2.json" with name "test-put" to endpoint "/api/test | content_type=json"
-
-    # same as
-    Then put request "test/request.j2.json" with name "test-put" to endpoint "/api/test"
-    And set response content type to "application/json"
-    ```
-
-    `content_type` will be removed from the actual `endpoint` value.
-
     Args:
         method (RequestMethod): type of request
         source (str): path to a template file relative to the directory `requests/`, which **must** exist in the directory the feature file is located
@@ -171,9 +151,15 @@ def step_task_request_file_with_name_endpoint(context: Context, method: RequestM
 
 @then(u'{method:Method} request "{source}" with name "{name}"')
 def step_task_request_file_with_name(context: Context, method: RequestMethod, source: str, name: str) -> None:
-    '''Creates a named request to the same endpoint as previous request, where the payload is defined in a template file.
+    '''
+    Creates an instance of the {@pylink grizzly.tasks.request} task, with the same `endpoint` as the previous Request task, where the
+    payload is defined in a template file.
 
-    ```gherkin
+    See {@pylink grizzly.tasks.request} task documentation for more information about arguments.
+
+    Example:
+
+    ``` gherkin
     Then post request "test/request1.j2.json" with name "test-post1" to endpoint "/api/test"
     Then post request "test/request2.j2.json" with name "test-post2"
 
@@ -182,18 +168,6 @@ def step_task_request_file_with_name(context: Context, method: RequestMethod, so
     Then post request "test/request2.j2.json" with name "test-post2" to endpoint "/api/test"
     ```
 
-    `endpoint` has support for setting response content type as a parameter:
-
-    ```gherkin
-    Then post request "test/request1.j2.json" with name "test-post1" to endpoint "/api/test | content_type=json"
-
-    # same as
-    Then post request "test/request1.j2.json" with name "test-post1" to endpoint "/api/test"
-    And set response content type to "application/json"
-    ```
-
-    `content_type` will be removed from the actual `endpoint` value.
-
     Args:
         method (RequestMethod): type of request
         source (str): path to a template file relative to the directory `requests/`, which **must** exist in the directory the feature file is located
@@ -207,12 +181,18 @@ def step_task_request_file_with_name(context: Context, method: RequestMethod, so
 
 @then(u'{method:Method} request with name "{name}"')
 def step_task_request_text_with_name(context: Context, method: RequestMethod, name: str) -> None:
-    '''Creates a named request to the same endpoint as previous request, where optional payload is defined directly in the feature file.
+    '''
+    Creates an instance of the {@pylink grizzly.tasks.request} task, where optional payload is defined directly in the feature file.
+
+    See {@pylink grizzly.tasks.request} task documentation for more information about arguments.
 
     If `method` in the expression is `post`, `put` or `send` the payload in the request **must** be defined directly in the feature file after the step.
     This step is useful if `method` and `endpoint` are the same as previous request, but the payload should be different.
 
-    ```gherkin
+    Example:
+
+    ``` gherkin
+    # example-1
     Then post request with name "test-post-1" to endpoint "/api/test"
         """
         {
@@ -226,7 +206,7 @@ def step_task_request_text_with_name(context: Context, method: RequestMethod, na
         }
         """
 
-    # same as
+    # same as example-1
     Then post request with name "test-post-1" to endpoint "/api/test"
         """
         {
@@ -240,10 +220,11 @@ def step_task_request_text_with_name(context: Context, method: RequestMethod, na
         }
         """
 
+    # example-2
     Then get request with name "test-get-1" from endpoint "/api/test"
     Then get request with name "test-get-2"
 
-    # same as
+    # same as example-2
     Then get request with name "test-get-1" from endpoint "/api/test"
     Then get request with name "test-get-2" from endpoint "/api/test"
     ```
@@ -263,17 +244,22 @@ def step_task_request_text_with_name(context: Context, method: RequestMethod, na
 
 @then(u'wait for "{wait_time:f}" seconds')
 def step_task_wait_seconds(context: Context, wait_time: float) -> None:
-    '''Create an explicit wait (task) in the scenario. The scenario will wait the time specified (seconds) on top
-    of what has been defined in `step_setup_wait_time`.
+    '''
+    Creates an instace of the {@pylink grizzly.tasks.wait} task. The scenario will wait the specified time (seconds) in
+    additional to the wait time specified by {@pylink grizzly.tasks.request_wait}.
 
-    ```gherkin
-    And wait time inbetween requests is random between "1.5" and "2.5" seconds
+    See {@pylink grizzly.tasks.wait} task documentation for more information about the task.
+
+    Example:
+
+    ``` gherkin
+    And wait "1.5..2.5" seconds between tasks
     ...
-    And wait for "1.5" seconds
+    Then wait for "1.5" seconds
     ```
 
-    Above combinations of steps will result in a wait time between 3 and 4 seconds for the first request that is defined after the
-    `And wait for...`-step.
+    Above combinations of steps will result in a wait time between 3 and 4 seconds for the first {@pylink grizzly.tasks} that is defined after the
+    `Then wait for...`-step.
 
     Args:
         wait_time (float): wait time in seconds
@@ -282,16 +268,22 @@ def step_task_wait_seconds(context: Context, wait_time: float) -> None:
 
     assert wait_time > 0.0, 'wait time cannot be less than 0.0 seconds'
 
-    grizzly.scenario.add_task(WaitTask(time=wait_time))
+    grizzly.scenario.tasks.add(WaitTask(time=wait_time))
 
 
-@then(u'print message "{message}"')
-def step_task_print_message(context: Context, message: str) -> None:
-    '''Print a message in the scenario. Useful for visualizing values of variables.
-    The message can be a jinja template, and any variables will be rendered at the time the task executes.
+@then(u'log message "{message}"')
+def step_task_log_message(context: Context, message: str) -> None:
+    '''
+    Creates an instance of the {@pylink grizzly.tasks.log_message} task. Prints a log message in the console, useful for troubleshooting values
+    of variables or set markers in log files.
 
-    ```gherkin
-    And print message "context_variable='{{ context_variable }}'
+    The message supports {@link framework.usage.variables.templating}. See {@pylink grizzly.tasks.log_message} task documentation for more
+    information about the task.
+
+    Example:
+
+    ``` gherkin
+    And log message "context_variable='{{ context_variable }}'
     ```
 
     Args:
@@ -299,16 +291,20 @@ def step_task_print_message(context: Context, message: str) -> None:
     '''
 
     grizzly = cast(GrizzlyContext, context.grizzly)
-    grizzly.scenario.add_task(PrintTask(message=message))
+    grizzly.scenario.tasks.add(LogMessage(message=message))
 
 
 @then(u'parse "{content}" as "{content_type:ContentType}" and save value of "{expression}" in variable "{variable}"')
 def step_task_transform(context: Context, content: str, content_type: TransformerContentType, expression: str, variable: str) -> None:
-    '''Parse (part of) a JSON object or a XML document and extract a specific value from that and save into a variable.
+    '''
+    Creates an instance of the {@pylink grizzly.tasks.transformer} task. Transforms the specified `content` with `content_type` to an
+    object that an transformer can extract information from with the specified `expression`.
 
-    This can be especially useful in combination with [`AtomicMessageQueue`](/grizzly/framework/usage/variables/testdata/messagequeue/) variable.
+    See {@pylink grizzly.tasks.transformer} task documentation for more information about the task.
 
-    ```gherkin
+    Example:
+
+    ``` gherkin
     And value for variable "document_id" is "None"
     And value for variable "document_title" is "None"
     And value for variable "document" is "{\"document\": {\"id\": \"DOCUMENT_8843-1\", \"title\": \"TPM Report 2021\"}}"
@@ -318,14 +314,15 @@ def step_task_transform(context: Context, content: str, content_type: Transforme
     ```
 
     Args:
-        contents (str): contents to parse, supports templating or a static string
+        contents (str): contents to parse, supports {@link framework.usage.variables.templating} or a static string
         content_type (TransformerContentType): MIME type of `contents`
         expression (str): JSON or XPath expression for specific value in `contents`
         variable (str): name of variable to save value to, must have been initialized
     '''
 
     grizzly = cast(GrizzlyContext, context.grizzly)
-    grizzly.scenario.add_task(TransformerTask(
+    grizzly.scenario.tasks.add(TransformerTask(
+        grizzly,
         content=content,
         content_type=content_type,
         expression=expression,
@@ -333,51 +330,58 @@ def step_task_transform(context: Context, content: str, content_type: Transforme
     ))
 
 
-@then(u'get "{endpoint}" and save response in "{variable}"')
-def step_task_client_get_endpoint(context: Context, endpoint: str, variable: str) -> None:
-    '''Get information from another host or endpoint than the scenario is load testing and save the response in a variable.
+@then(u'get "{endpoint}" with name "{name}" and save response in "{variable}"')
+def step_task_client_get_endpoint(context: Context, endpoint: str, name: str, variable: str) -> None:
+    '''
+    Creates an instance of a {@pylink grizzly.tasks.clients} task, actual implementation of the task is determined
+    based on the URL scheme specified in `endpoint`. Gets information from another host or endpoint than the scenario
+    is load testing and saves the response in a variable.
 
-    Task implementations are found in `grizzly.task.clients` and each implementation is looked up through the scheme in the
-    specified endpoint. If the endpoint is a variable, one have to manually specify the endpoint scheme even though the
-    resolved variable contains the scheme. In this case the manually specified scheme will be removed to the endpoint actually
-    used by the task.
+    See {@pylink grizzly.tasks.clients} task documentation for more information about client tasks.
 
-    ```gherkin
-    Then get "https://www.example.org/example.json" and save response in "example_openapi"
-    Then get "http://{{ endpoint }}" and save response in "endpoint_result"
+    Example:
+
+    ``` gherkin
+    Then get "https://www.example.org/example.json" with name "example-1" and save response in "example_openapi"
+    Then get "http://{{ endpoint }}" with name "example-2" and save response in "endpoint_result"
     ```
 
     Args:
         endpoint (str): information about where to get information, see the specific getter task implementations for more information
+        name (str): name of the request, used in request statistics
         variable (str): name of, initialized, variable where response will be saved in
     '''
     grizzly = cast(GrizzlyContext, context.grizzly)
 
     task_client = get_task_client(endpoint)
 
-    grizzly.scenario.add_task(task_client(
+    grizzly.scenario.tasks.add(task_client(
         RequestDirection.FROM,
         endpoint,
+        name,
         variable=variable,
     ))
 
 
-@then(u'put "{source}" to "{endpoint}" as "{destination}"')
-def step_task_client_put_endpoint_file_destination(context: Context, source: str, endpoint: str, destination: str) -> None:
-    '''Put information to another host or endpoint than the scenario is load testing, source being a file.
+@then(u'put "{source}" to "{endpoint}" with name "{name}" as "{destination}"')
+def step_task_client_put_endpoint_file_destination(context: Context, source: str, endpoint: str, name: str, destination: str) -> None:
+    '''
+    Creates an instance of a {@pylink grizzly.tasks.clients} task, actual implementation of the task is determined
+    based on the URL scheme specified in `endpoint`. Puts information, source being a file, to another host or endpoint than the scenario
+    is load testing and saves the response in a variable
 
-    Task implementations are found in `grizzly.task.clients` and each implementation is looked up through the scheme in the
-    specified endpoint. If the endpoint is a variable, one have to manually specify the endpoint scheme even though the
-    resolved variable contains the scheme. In this case the manually specified scheme will be removed to the endpoint actually
-    used by the task.
+    See {@pylink grizzly.tasks.clients} task documentation for more information about client tasks.
 
-    ```gherkin
-    Then put "test-file.json" to "bs://my-storage?AccountKey=aaaabbb=&Container=my-container" as "uploaded-test-file.json"
+    Example:
+
+    ``` gherkin
+    Then put "test-file.json" to "bs://my-storage?AccountKey=aaaabbb=&Container=my-container" with name "upload-file" as "uploaded-test-file.json"
     ```
 
     Args:
-        source (str): relative path to file in `feature/requests`, supports templating
+        source (str): relative path to file in `feature/requests`, supports {@link framework.usage.variables.templating}
         endpoint (str): information about where to get information, see the specific getter task implementations for more information
+        name (str): name of the request, used in request statistics
         destination (str): name of source on the destination
     '''
     assert context.text is None, 'step text is not allowed for this step expression'
@@ -388,21 +392,65 @@ def step_task_client_put_endpoint_file_destination(context: Context, source: str
 
     assert not is_template(source), 'source file cannot be a template'
 
-    grizzly.scenario.add_task(task_client(
+    grizzly.scenario.tasks.add(task_client(
         RequestDirection.TO,
         endpoint,
+        name,
         source=source,
         destination=destination,
     ))
 
 
+@then(u'put "{source}" to "{endpoint}" with name "{name}"')
+def step_task_client_put_endpoint_file(context: Context, source: str, endpoint: str, name: str) -> None:
+    '''
+    Creates an instance of a {@pylink grizzly.tasks.clients} task, actual implementation of the task is determined
+    based on the URL scheme specified in `endpoint`. Puts information, source being a file, to another host or endpoint than the scenario
+    is load testing and saves the response in a variable
+
+    See {@pylink grizzly.tasks.clients} task documentation for more information about client tasks.
+
+    Example:
+
+    ``` gherkin
+    Then put "test-file.json" to "bs://my-storage?AccountKey=aaaabbb=&Container=my-container" with name "upload-file"
+    ```
+
+    Args:
+        source (str): relative path to file in `feature/requests`, supports {@link framework.usage.variables.templating}
+        endpoint (str): information about where to get information, see the specific getter task implementations for more information
+        name (str): name of the request, used in request statistics
+    '''
+    assert context.text is None, 'step text is not allowed for this step expression'
+
+    grizzly = cast(GrizzlyContext, context.grizzly)
+
+    task_client = get_task_client(endpoint)
+
+    assert not is_template(source), 'source file cannot be a template'
+
+    grizzly.scenario.tasks.add(task_client(
+        RequestDirection.TO,
+        endpoint,
+        name,
+        source=source,
+        destination=None,
+    ))
+
+
 @then(u'parse date "{value}" and save in variable "{variable}"')
 def step_task_date(context: Context, value: str, variable: str) -> None:
-    '''Parses a datetime string and transforms it according to specified arguments.
+    '''
+    Creates an instance of the {@pylink grizzly.tasks.date} task. Parses a datetime string and transforms it according to specified
+    arguments.
+
+    See {@pylink grizzly.tasks.date} task documentation for more information about arguments.
 
     This step is useful when changes has to be made to a datetime representation during an iteration of a scenario.
 
-    ```gherkin
+    Example:
+
+    ``` gherkin
     ...
     And value for variable "date1" is "none"
     And value for variable "date2" is "none"
@@ -417,21 +465,11 @@ def step_task_date(context: Context, value: str, variable: str) -> None:
     Args:
         value (str): datetime string and arguments
         variable (str): name of, initialized, variable where response will be saved in
-
-    ## Arguments
-
-    At least one of the following optional arguments **must** be specified:
-
-    * `format` _str_ - a python [`strftime` format string](https://docs.python.org/3/library/datetime.html#strftime-and-strptime-format-codes), this argument is required
-
-    * `timezone` _str_ (optional) - a valid [timezone name](https://en.wikipedia.org/wiki/List_of_tz_database_time_zones)
-
-    * `offset` _str_ (optional) - a time span string describing the offset, Y = years, M = months, D = days, h = hours, m = minutes, s = seconds, e.g. `1Y-2M10D`
     '''
     grizzly = cast(GrizzlyContext, context.grizzly)
     assert variable in grizzly.state.variables, f'variable {variable} has not been initialized'
 
-    grizzly.scenario.add_task(DateTask(
+    grizzly.scenario.tasks.add(DateTask(
         value=value,
         variable=variable,
     ))
@@ -439,13 +477,24 @@ def step_task_date(context: Context, value: str, variable: str) -> None:
 
 @given(u'an async request group with name "{name}"')
 def step_task_async_group_start(context: Context, name: str) -> None:
-    '''Creates a group of requests that should be executed asynchronously. All requests tasks created after this step will be added to the
+    '''
+    Creates an instance of the {@pylink grizzly.tasks.async_group} task. All {@pylink grizzly.tasks.request} tasks created after this step will be added to the
     request group, until the group is closed.
 
-    ```gherkin
+    See {@pylink grizzly.tasks.async_group} task documentation for more information.
+
+    Example:
+
+    ``` gherkin
     Given an async request group with name "async-group-1"
-    Then put "test-file.json" to "bs://my-storage?AccountKey=aaaabbb=&Container=my-container" as "uploaded-test-file.json"
-    Then get "https://www.example.org/example.json" and save response in "example_openapi"
+    Then post request with name "test-post-2" to endpoint "/api/test"
+        """
+        {
+            "value": "i have good news!"
+        }
+        """
+
+    Then get request with name "test-get-1" from endpoint "/api/test"
     And close async request group
     ```
 
@@ -463,14 +512,24 @@ def step_task_async_group_start(context: Context, name: str) -> None:
 
 @then(u'close async request group')
 def step_task_async_group_close(context: Context) -> None:
-    '''Closes an open async request group, to end the "boxing" of requests that should run asynchronously.
+    '''
+    Closes the instance created in {@pylink grizzly.steps.scenario.tasks.step_task_async_group_start}, and adds the {@pylink grizzly.tasks.async_group} task to the list of tasks
+    that the scenario is going to execute.
 
-    Must be preceeded by the expression `Given an async request group with name..." expression, and one or more requests expressions.
+    See {@pylink grizzly.tasks.async_group} task documentation for more information.
 
-    ```gherkin
+    Example:
+
+    ``` gherkin
     Given an async request group with name "async-group-1"
-    Then put "test-file.json" to "bs://my-storage?AccountKey=aaaabbb=&Container=my-container" as "uploaded-test-file.json"
-    Then get "https://www.example.org/example.json" and save response in "example_openapi"
+    Then post request with name "test-post-2" to endpoint "/api/test"
+        """
+        {
+            "value": "i have good news!"
+        }
+        """
+
+    Then get request with name "test-get-1" from endpoint "/api/test"
     And close async request group
     ```
     '''
@@ -479,5 +538,117 @@ def step_task_async_group_close(context: Context) -> None:
     assert grizzly.scenario.async_group is not None, 'no async request group is open'
     assert len(grizzly.scenario.async_group.requests) > 0, f'there are no requests in async group "{grizzly.scenario.async_group.name}"'
 
-    grizzly.scenario.add_task(grizzly.scenario.async_group)
+    grizzly.scenario.tasks.add(grizzly.scenario.async_group)
     grizzly.scenario.async_group = None
+
+
+@then(u'start timer with name "{name}"')
+def step_task_timer_start(context: Context, name: str) -> None:
+    '''
+    Creates an instance of the {@pylink grizzly.tasks.timer} task. Starts a timer to measure the "request time" for all tasks between
+    the start and stop of the timer.
+
+    See {@pylink grizzly.tasks.timer} task documentation for more information.
+
+    Example:
+
+    ``` gherkin
+    Then start timer with name "parsing-xml"
+    ...
+    And stop timer with name "parsing-xml"
+    ```
+    '''
+    grizzly = cast(GrizzlyContext, context.grizzly)
+    assert name not in grizzly.scenario.timers, f'timer with name {name} has already been defined'
+
+    task = TimerTask(name=name)
+
+    grizzly.scenario.timers.update({
+        name: task,
+    })
+
+    grizzly.scenario.tasks.add(task)
+
+
+@then(u'stop timer with name "{name}"')
+def step_task_timer_stop(context: Context, name: str) -> None:
+    '''
+    Adds the instance created by {@pylink grizzly.steps.scenario.tasks.step_task_timer_start} to the list of scenario tasks.
+
+    See {@pylink grizzly.tasks.timer} task documentation for more information.
+
+    Example:
+
+    ``` gherkin
+    Then start timer with name "parsing-xml"
+    ...
+    And stop timer with name "parsing-xml"
+    ```
+    '''
+    grizzly = cast(GrizzlyContext, context.grizzly)
+
+    task = grizzly.scenario.timers.get(name, None)
+
+    assert task is not None, f'timer with name {name} has not been defined'
+
+    grizzly.scenario.tasks.add(task)
+    grizzly.scenario.timers.update({name: None})
+
+
+@given(u'wait "{min_time:g}..{max_time:g}" seconds between tasks')
+def step_task_request_wait_between(context: Context, min_time: float, max_time: float) -> None:
+    '''
+    Creates an instance of the {@pylink grizzly.tasks.request_wait} task. Sets number of, randomly, seconds the {@pylink grizzly.users}
+    will wait between executing each task.
+
+    See {@pylink grizzly.tasks.request_wait} task documentation for more information.
+
+    Example:
+
+    ``` gherkin
+    And wait "1.4..1.7" seconds between tasks
+    # wait between 1.4 and 1.7 seconds
+    Then get request with name "test-get-1" from endpoint "..."
+    # wait between 1.4 and 1.7 seconds
+    Then get request with name "test-get-2" from endpoint "..."
+    # wait between 1.4 and 1.7 seconds
+    And wait "0.1" seconds between tasks
+    # wait 0.1 seconds
+    Then get request with name "test-get-3" from endpoint "..."
+    # wait 0.1 seconds
+    ...
+    ```
+    '''
+    grizzly = cast(GrizzlyContext, context.grizzly)
+    if min_time > max_time:
+        min_time, max_time = max_time, min_time
+
+    grizzly.scenario.tasks.add(RequestWaitTask(min_time=min_time, max_time=max_time))
+
+
+@given(u'wait "{time:g}" seconds between tasks')
+def step_task_request_wait_constant(context: Context, time: float) -> None:
+    '''
+    Creates an instance of the {@pylink grizzly.tasks.request_wait} task. Sets number of, constant, seconds the {@pylink grizzly.users}
+    will wait between executing each task.
+
+    See {@pylink grizzly.tasks.request_wait} task documentation for more information.
+
+    Example:
+
+    ``` gherkin
+    And wait "1.4" seconds between tasks
+    # wait 1.4 seconds
+    Then get request with name "test-get-1" from endpoint "..."
+    # wait 1.4 seconds
+    Then get request with name "test-get-2" from endpoint "..."
+    # wait 1.4 seconds
+    And wait "0.1" seconds between tasks
+    # wait 0.1 seconds
+    Then get request with name "test-get-3" from endpoint "..."
+    # wait 0.1 seconds
+    ...
+    ```
+    '''
+    grizzly = cast(GrizzlyContext, context.grizzly)
+    grizzly.scenario.tasks.add(RequestWaitTask(time))
