@@ -1,3 +1,20 @@
+'''
+@anchor pydoc:grizzly.tasks Tasks
+Tasks are functionality that is executed by `locust` at run time as they are specified in the feature file.
+
+The most essential task is {@pylink grizzly.tasks.request}, which all {@pylink grizzly.users} is using to make
+requests to the endpoint that is being load tested.
+
+All other tasks are helper tasks for things that needs to happen after or before a {@pylink grizzly.tasks.request}, stuff like extracting information from
+a previous response or fetching additional test data from a different endpoint ("{@link pydoc:grizzly.tasks.clients}").
+
+## Custom
+
+It is possible to implement custom tasks, the only requirement is that they inherit `grizzly.tasks.GrizzlyTask`. To get them to be executed by `grizzly`,
+a step implementation is also needed.
+
+There are examples of this in the {@link framework.example}.
+'''
 from abc import ABC
 from typing import TYPE_CHECKING, Any, Callable, List, Type, Set, Optional
 from os import environ
@@ -27,9 +44,12 @@ class GrizzlyTask(ABC):
         def is_template(value: str) -> bool:
             return '{{' in value and '}}' in value
 
+        # tasks with no @template decorator
+        if not hasattr(self, '__template_attributes__'):
+            return []
+
         templates: Set[str] = set()
-        attributes = getattr(self, '__template_attributes__', [])
-        for attribute in attributes:
+        for attribute in self.__template_attributes__:
             value = getattr(self, attribute, None)
             if value is None:
                 continue
@@ -47,12 +67,18 @@ class GrizzlyTask(ABC):
                     templates.add(value)
             elif isinstance(value, list):
                 for list_value in value:
-                    if is_template(list_value):
+                    if isinstance(list_value, GrizzlyTask):
+                        templates.update(list_value.get_templates())
+                    elif is_template(list_value):
                         templates.add(list_value)
             elif isinstance(value, dict):
                 for dict_value in value.values():
-                    if is_template(dict_value):
+                    if isinstance(dict_value, GrizzlyTask):
+                        templates.update(dict_value.get_templates())
+                    elif is_template(dict_value):
                         templates.add(dict_value)
+            elif isinstance(value, GrizzlyTask):
+                templates.update(value.get_templates())
 
         return list(templates)
 
@@ -65,30 +91,33 @@ class template:
         if len(additional_attributes) > 0:
             self.attributes += list(additional_attributes)
 
-    def __call__(self, task: Type[GrizzlyTask]) -> Type[GrizzlyTask]:
+    def __call__(self, cls: Type[GrizzlyTask]) -> Type[GrizzlyTask]:
+        cls.__template_attributes__ = self.attributes
 
-        setattr(task, '__template_attributes__', self.attributes)
-
-        return task
+        return cls
 
 
 from .request import RequestTask, RequestTaskHandlers, RequestTaskResponse
 from .wait import WaitTask
-from .print import PrintTask
+from .log_message import LogMessage
 from .transformer import TransformerTask
 from .until import UntilRequestTask
 from .date import DateTask
 from .async_group import AsyncRequestGroupTask
+from .timer import TimerTask
+from .request_wait import RequestWaitTask
 
 
 __all__ = [
     'RequestTaskHandlers',
     'RequestTaskResponse',
     'RequestTask',
-    'PrintTask',
+    'LogMessage',
     'WaitTask',
     'TransformerTask',
     'UntilRequestTask',
     'DateTask',
     'AsyncRequestGroupTask',
+    'TimerTask',
+    'RequestWaitTask',
 ]
