@@ -1,22 +1,31 @@
-'''This task runs all requests in the group asynchronously.
+'''
+@anchor pydoc:grizzly.tasks.async_group Async Group
+This task runs all requests in the group asynchronously.
 
-The name of all request added to the group will be prefixed with async group `<name>:`. Request type for this task is `ASYNC`.
-
-Arguments:
-
-* `name` (str): name of the group of asynchronously requests
-
-Instances of this task is created with step expressions:
-
-* [`step_task_async_group_start`](/grizzly/framework/usage/steps/scenario/tasks/#step_task_async_group_start)
-
-* [`step_task_async_group_close`](/grizzly/framework/usage/steps/scenario/tasks/#step_task_async_group_close)
-
-Requests are added to the group with the same step expressions as [`RequestTask`](/grizzly/framework/usage/tasks/request/).
+The name of requests added to the group will be prefixed with async group `<name>:`
 
 Enable `gevent` debugging for this task by running with argument `--verbose` and setting environment variable `GEVENT_MONITOR_THREAD_ENABLE`.
+
+## Step implementations
+
+* {@pylink grizzly.steps.scenario.tasks.step_task_async_group_start}
+
+* {@pylink grizzly.steps.scenario.tasks.step_task_async_group_close}
+
+Requests are added to the group with the same step implementations as {@pylink grizzly.tasks.request} task.
+
+## Statistics
+
+Executions of this task will be visible in `locust` request statistics with request type `ASYNC`. `name` will be suffixed with ` (<n>)`,
+where `<n>` is the number of requests in the group. Each request in the group will have it's own entry in the statistics as an ordinary
+{@pylink grizzly.tasks.request} task.
+
+## Arguments
+
+* `name` (str): name of the group of asynchronously requests
 '''
 import logging
+import inspect
 
 from os import environ
 
@@ -25,16 +34,16 @@ import gevent
 from typing import TYPE_CHECKING, Any, Callable, List, Optional, Tuple
 from time import perf_counter as time_perf_counter
 
-from . import GrizzlyTask, template
-from .request import RequestTask
+from . import GrizzlyTask, RequestTask, template
 from ..users.base import AsyncRequests
+from ..types import RequestType
 
 if TYPE_CHECKING:  # pragma: no cover
     from ..context import GrizzlyContextScenario
     from ..scenarios import GrizzlyScenario
 
 
-@template('name')
+@template('name', 'requests')
 class AsyncRequestGroupTask(GrizzlyTask):
     requests: List[RequestTask]
 
@@ -49,8 +58,6 @@ class AsyncRequestGroupTask(GrizzlyTask):
         self.requests.append(request)
 
     def __call__(self) -> Callable[['GrizzlyScenario'], Any]:
-        import inspect
-
         def task(parent: 'GrizzlyScenario') -> Any:
             if not isinstance(parent.user, AsyncRequests):
                 raise NotImplementedError(f'{parent.user.__class__.__name__} does not inherit AsyncRequests')
@@ -112,7 +119,7 @@ class AsyncRequestGroupTask(GrizzlyTask):
                 response_time = int((time_perf_counter() - start) * 1000)
 
                 parent.user.environment.events.request.fire(
-                    request_type='ASYNC',
+                    request_type=RequestType.ASYNC_GROUP(),
                     name=f'{self.scenario.identifier} {self.name} ({len(self.requests)})',
                     response_time=response_time,
                     response_length=response_length,
