@@ -1,28 +1,31 @@
 # pylint: disable=line-too-long
-'''Listens for messages on Azure Service Bus queue or topic.
+'''
+@anchor pydoc:grizzly.testdata.variables.servicebus Servicebus
+Listens for messages on Azure Service Bus queue or topic.
 
-Use [transformer task](/grizzly/framework/usage/tasks/transformer/) to extract specific parts of the message.
+Use {@pylink grizzly.tasks.transformer} task to extract specific parts of the message.
 
 ## Format
 
 Initial value for a variable must have the prefix `queue:` or `topic:` followed by the name of the targeted
 type. When receiving messages from a topic, the argument `subscription:` is mandatory. The format of endpoint is:
 
-```plain
+``` plain
 [queue|topic]:<endpoint name>[, subscription:<subscription name>][, expression:<expression>]
 ```
 
 Where `<expression>` can be a XPath or jsonpath expression, depending on the specified content type. This argument is only allowed when
 receiving messages. See example below.
 
-> **Warning**: Do not use `expression` to filter messages unless you do not care about the messages that does not match the expression. If
-> you do care about them, you should setup a subscription to do the filtering in Azure.
+!!! attention
+    Do not use `expression` to filter messages unless you do not care about the messages that does not match the expression. If
+    you do care about them, you should setup a subscription to do the filtering in Azure.
 
-Arguments support templating for their value, but not the complete endpoint value.
+Arguments support {@link framework.usage.variables.templating} for their value, but not the complete endpoint value.
 
 Examples:
 
-```plain
+``` plain
 queue:test-queue
 topic:test-topic, subscription:test-subscription
 queue:"$conf::sb.endpoint.queue"
@@ -39,13 +42,13 @@ queue:"{{ queue_name }}", expression="$.document[?(@.name=='TPM report')]"
 
 ### URL format
 
-```plain
+``` plain
 [Endpoint=]sb://<hostname>/;SharedAccessKeyName=<shared key name>;SharedAccessKey=<shared key>
 ```
 
-The complete `url` has templating support, but not parts of it.
+The complete `url` has {@link framework.usage.variables.templating} support, but not parts of it.
 
-```plain
+``` plain
 # valid
 $conf::sb.url
 
@@ -55,8 +58,8 @@ Endpoint=sb://$conf::sb.hostname/;SharedAccessKeyName=$conf::sb.keyname;SharedAc
 
 ## Example
 
-```gherkin
-And value of variable "AtomicServiceBus.document_id" is "queue:documents-in | wait=120, url=$conf::sb.endpoint, repeat=True"
+``` gherkin
+And value for variable "AtomicServiceBus.document_id" is "queue:documents-in | wait=120, url=$conf::sb.endpoint, repeat=True"
 ...
 Given a user of type "RestApi" load testing "http://example.com"
 ...
@@ -74,8 +77,8 @@ When specifying an expression, the messages on the endpoint is first peeked on.
 If no matching messages was found when peeking, it is repeated again up until the specified `wait` seconds has elapsed. To use expression, a content type must
 be specified for the endpint, e.g. `application/xml`.
 
-```gherking
-And value of variable "AtomicServiceBus.tpm_document" is "queue:documents-in | wait=120, url=$conf::sb.endpoint, repeat=True, content_type=json, expression='$.document[?(@.name=='TPM Report')'"
+``` gherkin
+And value for variable "AtomicServiceBus.tpm_document" is "queue:documents-in | wait=120, url=$conf::sb.endpoint, repeat=True, content_type=json, expression='$.document[?(@.name=='TPM Report')'"
 ```
 '''  # noqa: E501
 import logging
@@ -92,9 +95,10 @@ from grizzly_extras.async_message import AsyncMessageContext, AsyncMessageReques
 from grizzly_extras.arguments import split_value, parse_arguments, get_unsupported_arguments
 from grizzly_extras.transformer import TransformerContentType
 
-from ...types import AtomicVariable, bool_typed
+from ...types import RequestType, bool_typed
 from ...context import GrizzlyContext
 from ..utils import resolve_variable
+from . import AtomicVariable
 
 
 def atomicservicebus__base_type__(value: str) -> str:
@@ -340,7 +344,7 @@ class AtomicServiceBus(AtomicVariable[str]):
         response: Optional[AsyncMessageResponse] = None
         request: AsyncMessageRequest = {
             'worker': settings['worker'],
-            'action': 'HELLO',
+            'action': RequestType.HELLO.name,
             'context': context,
         }
 
